<?php

// 

define('PATH_TO_ROOT', '../../../../..');
define('DS', DIRECTORY_SEPARATOR);
define('ROOT', realpath(PATH_TO_ROOT));
define('HTTP_ROOT', $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . dirname($_SERVER['SCRIPT_NAME']) . '/' . PATH_TO_ROOT);

function extractFiles($fso, $path) {
    if (!is_array($fso['items'])) {
        return;
    }

    if (!file_exists($path)) {
        mkdir($path, 0777, true);
    }

    if (!is_writable($path)) {
        throw new Exception('Permission denied: ' . $path);
    }

    foreach ($fso['items'] as $name => $file) {
        if (isset($file['content']) && isset($file['type'])) {
            if ($file['content'] === '[DELETED]') {
                @unlink($path . DS . $name);
            } else {
                switch ($file['type']) {
                    case 'text':
                        file_put_contents($path . DS . $name, $file['content']);
                        break;
                    case 'data':
                        file_put_contents($path . DS . $name, base64_decode($file['content']));
                        break;
                }
            }
        } else if (isset($file['items']) && isset($file['type'])) {
            extractFiles($file, $path . DS . $name);
        }
    }
}

function uploadChunk($uploadPath)
{
    $result = array();

    $contentRange = getHeader('Content-Range');
    $isLast = getParam('last');

    if (!isset($_FILES['chunk']) || !file_exists($_FILES['chunk']['tmp_name'])) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty chunk data'
        );
    } else if (!$contentRange && !$isLast) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty Content-Range header'
        );
    } else {
        $rangeBegin = 0;

        if ($contentRange) {
            $contentRange = str_replace('bytes ', '', $contentRange);
            list($range, $total) = explode('/', $contentRange);
            list($rangeBegin, $rangeEnd) = explode('-', $range);
        }

        $tmpPath = $uploadPath . '.upload';
        @mkdir(dirname($tmpPath), 0777, true);

        $f = fopen($tmpPath, 'c');

        if (flock($f, LOCK_EX)) {
            fseek($f, (int) $rangeBegin);
            fwrite($f, file_get_contents($_FILES['chunk']['tmp_name']));

            flock($f, LOCK_UN);
            fclose($f);
        } else {
            throw new Exception('Permission denied: ' . $tmpPath);
        }

        if ($isLast) {
            if (file_exists($uploadPath)) {
                @unlink($uploadPath);
            }

            @mkdir(dirname($uploadPath), 0776, true);
            if (!is_writable(dirname($uploadPath))) {
                throw new Exception('Permission denied: ' . $uploadPath);
            }

            rename($tmpPath, $uploadPath);

            $result = array(
                'status' => 'done'
            );
        } else {
            $result['status'] = 'processed';
        }
    }

    return $result;
}

function rm_r($dir, $exclude = [], $deleteRoot = false)
{
    if (false === file_exists($dir)) {
        return false;
    }

    $iterator = new RecursiveIteratorIterator(
        new RecursiveCallbackFilterIterator(
            new RecursiveDirectoryIterator($dir, RecursiveDirectoryIterator::SKIP_DOTS),
            function ($current) use ($exclude) {
                return !in_array($current->getFilename(), $exclude);
            }
        ),
        RecursiveIteratorIterator::CHILD_FIRST
    );

    foreach ($iterator as $fileinfo) {
        if ($fileinfo->isDir()) {
            if (false === rmdir($fileinfo->getRealPath())) {
                return false;
            }
        } else {
            if (false === unlink($fileinfo->getRealPath())) {
                return false;
            }
        }
    }

    $iterator = null;

    if ($deleteRoot) {
        return rmdir($dir);
    }

    return true;
}

function cp_r($source, $dest, $exclude = [])
{
    @mkdir($dest, 0777, true);

    $iterator = new RecursiveIteratorIterator(
        new RecursiveCallbackFilterIterator(
            new RecursiveDirectoryIterator($source, RecursiveDirectoryIterator::SKIP_DOTS),
            function ($current) use ($exclude) {
                return !in_array($current->getFilename(), $exclude);
            }
        ),
        RecursiveIteratorIterator::SELF_FIRST
    );

    foreach ($iterator as $item) {
        if ($item->isDir()) {
            @mkdir($dest . DS . $iterator->getSubPathName(), 0777);
        } else {
            copy($item, $dest . DS . $iterator->getSubPathName());
        }
    }

    $iterator = null;
}

function dispatch()
{
    $action = empty($_REQUEST['action']) ? '' : preg_replace('/[^a-z]/i', '', $_REQUEST['action']);

    if (function_exists($action . 'Action')) {
        try {
            $result = call_user_func($action . 'Action');
        } catch (Exception $e) {
            $result = ['status' => 'error', 'message' => $e->getMessage()];
        }
    } else {
        $result = ['status' => 'error', 'message' => 'no action'];
    }

    if (is_array($result)) {
        echo json_encode($result);
    }
}

function getParam($name, $default = null)
{
    return empty($_REQUEST[$name]) ? $default : $_REQUEST[$name];
}

function getPost($name, $default = null)
{
    return empty($_POST[$name]) ? $default : $_POST[$name];
}

function getHeader($header)
{
    $temp = 'HTTP_' . strtoupper(str_replace('-', '_', $header));
    if (isset($_SERVER[$temp])) {
        return $_SERVER[$temp];
    }

    return null;
}

function getAvailableThemeName($path, $themeName)
{
    while (file_exists($path . DS . $themeName)) {
        preg_match('#(.*?)(\d{0,4})$#', $themeName, $m);
        $themeName = $m[1];
        $suffix = (int) $m[2];
        $suffix++;
        $themeName .= $suffix;
    };

    return $themeName;
}

class ZipHelper
{
    public static function zip($source, $wrapDir = '', $exclude = []) {
        $iterator = new RecursiveIteratorIterator(
            new RecursiveCallbackFilterIterator(
                new RecursiveDirectoryIterator($source, RecursiveDirectoryIterator::SKIP_DOTS | FilesystemIterator::UNIX_PATHS),
                function ($current) use ($exclude) {
                    return !in_array($current->getFilename(), $exclude);
                }
            ),
            RecursiveIteratorIterator::SELF_FIRST
        );

        $outPath = tempnam('tmp', 'zip');
        $z = new ZipArchive();
        $z->open($outPath, ZipArchive::CREATE);

        foreach ($iterator as $item) {
            if (!$item->isDir()) {
                $z->addFile($item, $wrapDir . '/' . $iterator->getSubPathName());
            }
        }

        $iterator = null;

        $z->close();
        return $outPath;
    }

    public static function unzip($file, $target) {
        $zip = new ZipArchive;
        $res = $zip->open($file);
        @mkdir($target, 0777, true);

        if ($res === true) {
            for($i = 0; $i < $zip->numFiles; $i++) {
                $data = $zip->getFromIndex($i);
                $filename = preg_replace('#[/\\\]#', DS, $zip->getNameIndex($i));
                $dest = $target . DS . $filename;

                if (substr($dest, -1) !== DS) {
                    @mkdir(dirname($dest), 0777, true);
                    file_put_contents($dest, $data);
                }
            }

            $zip->close();
            return ['status' => 'done'];
        } else {
            return ['status' => 'error', 'message' => 'Unzip status ' . $res];
        }
    }
}

//