/* exported DataProvider */
/* global __app, FileManager, DataProviderHelper */

// 

var DataProvider = {};

(function () {
    'use strict';

    DataProvider.load = function load() {
        return __app.project.data;
    };

    DataProvider.doExport = function doExport(data, callback) {
        __app.request('export', {
            fso: JSON.stringify(__app.run(data)),
            theme: __app.project.name,
            method: data.projectData ? 'save' : 'update'
        }).done(function () {
            callback(null);
        }).fail(function (xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.save = function save(data, callback) {
        DataProvider.doExport(data, callback);
    };

    DataProvider.getMd5Hashes = function getMd5Hashes() {
        return __app.project.hashes;
    };

    DataProvider.getAllCssJsSources = function getAllCssJsSources() {
        return __app.project.cache;
    };

    DataProvider.updatePreviewTheme = function updatePreviewTheme(callback) {
        __app.request('start').done(function () {
            callback(null);
        }).fail(function getThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.makeThemeAsActive = function makeThemeAsActive(callback, theme) {
        theme = theme || '';
        callback(null);
    };

    DataProvider.backToAdmin = function backToAdmin() {
        window.location = __app.config.url.admin;
    };

    DataProvider.getMaxRequestSize = function getMaxRequestSize() {
        return __app.config.env.maxRequestSize;
    };

    DataProvider.getVersion = function getVersion() {
        return '0.0.2';
    };

    DataProvider.reloadThemesInfo = function reloadThemesInfo(callback) {
        __app.request('themes').done(function (response) {
            callback(null, JSON.stringify({themes: response.themes}));
        }).fail(function (xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.getInfo = function getInfo() {
        return {
            cmsName: 'Html',
            cmsVersion: __app.config.version,
            adminPage: __app.config.url.admin,
            startPage: __app.config.url.preview,
            templates: __app.project.model.getUsedTemplates(),
            thumbnails: [{name: 'preview.png', width: 200, height: 200}],
            isThemeActive: __app.project.active,
            themeName: __app.project.name,
            uploadImage: __app.backendUrl('image'),
            uploadTheme: __app.backendUrl('theme'),
            themeArchiveExt: 'zip',
            themes: __app.config.themes,
            pathToManifest: __app.config.path.editor + '/themler.manifest'
        };
    };

    DataProvider.canRename = function canRename(themeName, callback) {
        __app.request('canRename', {
            themeName: themeName
        }).done(function (response) {
            callback(null, response.canRename);
        }).fail(function getThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.rename = function rename(targetName, callback) {
        __app.request('rename', {
            source: __app.project.name,
            target: targetName
        }).done(function () {
            callback(null);
        }).fail(function getThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.renameTheme = function renameTheme(sourceName, targetName, callback) {
        __app.request('rename', {
            source: sourceName,
            target: targetName
        }).done(function () {
            callback(null);
        }).fail(function getThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.removeTheme = function removeTheme(sourceName, callback) {
        __app.request('rename', {
            source: sourceName,
            target: ''
        }).done(function () {
            callback(null);
        }).fail(function getThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.copyTheme = function copyTheme(sourceName, targetName, callback) {
        __app.request('copy', {
            source: sourceName,
            target: targetName
        }).done(function () {
            callback(null);
        }).fail(function getThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.getTheme = function getTheme(params, callback) {
        var data = {
            sourceName: params.sourceName || '',
            targetName: params.targetName || params.themeName || '',
            editor: !!params.includeEditor
        };

        __app.request('zip', data, 'text').done(function (response) {
            callback(null, response);
        }).fail(function getThemeRequestFail(xhr, status, message) {
            callback(new Error(message));
        });
    };

    DataProvider.downloadTheme = function downloadTheme(themeName, callback) {
        DataProvider.getTheme({
            sourceName: themeName,
            includeEditor: true
        }, callback);
    };

    DataProvider.setFiles = function setFiles(files, callback) {
        var manifestPath = DataProvider.getInfo().pathToManifest;
        if (files[manifestPath]) {
            var fso = new FileManager();
            fso.write(manifestPath, files[manifestPath]);

            DataProvider.doExport({
                themeFso: fso,
                projectData: __app.project.data
            }, callback);
        } else {
            callback(null);
        }
    };

    DataProvider.getPosts = function getPosts(searchObj, callback) {
        if (!callback || typeof callback !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }
        if (!searchObj || typeof searchObj !== 'object') {
            throw DataProviderHelper.getResultError('Invalid search object');
        }

        var posts = __app.project.model.search(searchObj).map(function (entity) {
            return __app.project.model.getEntityData(entity);
        });

        callback(null, {logs: null, data: posts});
    };

    DataProvider.updatePost = function updatePost(postType, post, callback) {
        if (!callback || typeof callback !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }
        if (!post || typeof post !== 'object' || !post.id) {
            throw DataProviderHelper.getResultError('Invalid post object');
        }

        var updated = __app.project.model.updateEntity(post);

        callback(null, {logs: null, data: __app.project.model.getEntityData(updated)});
    };

    DataProvider.removePost = function removePost(postType, key, recursive, callback) {
        var list = __app.project.model.removeEntity(key, recursive);
        callback(null, {logs: null, data: list});
    };

    DataProvider.copyPost = function copyPost(postType, data, recursive, callback) {
        var list = __app.project.model.cloneEntity(data, recursive);
        callback(null, {logs: null, data: list});
    };

    DataProvider.newPost = function newPost(postType, data, callback) {
        var newEntity = __app.project.model.newEntity(postType, data);
        callback(null, {logs: null, data: __app.project.model.getEntityData(newEntity)});
    };

})();

//