/* global __html, FileManager, LogController */

// 

__html.Application = (function () {
    'use strict';

    function Application(config) {
        this.config = config;
        Object.freeze(this.config.path.assets);
        Object.freeze(this.config.path);
        var themeName = window.location.pathname.replace(/.*?([^\/]+)\/+__editor__\/+app.html/, '$1');
        this.project = new __html.Project(themeName, config);
        this.builder = new __html.Builder(config);
    }

    Application.prototype.run = function run(data) {
        LogController.time('[CMS] html app');

        var fso = data.themeFso || new FileManager();
        var builtResult = fso.enumerate('');

        LogController.time('[CMS] build media');
        this.builder.buildMedia(fso, data);
        LogController.timeEnd('[CMS] build media');

        LogController.time('[CMS] fso internals');
        builtResult.forEach(function (file) {
            var info = fileInfo(file);

            if (info.ext === 'html') {
                LogController.time('[CMS] update template: ' + file);
                this.project.updateTemplate(fso, file);
                LogController.timeEnd('[CMS] update template: ' + file);
            }

            if (['png', 'jpg', 'jpeg', 'gif'].indexOf(info.ext) !== -1) {
                LogController.time('[CMS] build image: ' + file);
                this.builder.buildImages(fso, file);
                LogController.timeEnd('[CMS] build image: ' + file);
            }

            if (['css'].indexOf(info.ext) !== -1) {
                LogController.time('[CMS] build css: ' + file);
                this.builder.buildCss(fso, file);
                LogController.timeEnd('[CMS] build css: ' + file);
            }

            if (['js'].indexOf(info.ext) !== -1) {
                LogController.time('[CMS] build js: ' + file);
                this.builder.buildJs(fso, file);
                LogController.timeEnd('[CMS] build js: ' + file);
            }

            if (file === this.config.path.content) {
                LogController.time('[CMS] update page content');
                this.project.updateContents(fso, file);
                LogController.timeEnd('[CMS] update page content');
            }

            if (file === this.config.path.editor) {
                /* expects manifest or app.html in __editor__ */
                LogController.time('[CMS] update app');
                this.project.updateApp(fso);
                LogController.timeEnd('[CMS] update app');

                LogController.time('[CMS] build app');
                this.builder.buildApp(fso, this.project);
                LogController.timeEnd('[CMS] build app');
            } else {
                fso.remove(file);
            }
        }, this);
        LogController.timeEnd('[CMS] fso internals');

        if (data.projectData) {
            LogController.time('[CMS] build project');
            this.project.updateProjectData(data);
            this.builder.buildProject(fso, this.project);
            LogController.timeEnd('[CMS] build project');
        }

        LogController.time('[CMS] build site');
        this.builder.buildSite(fso, this.project);
        LogController.timeEnd('[CMS] build site');

        LogController.timeEnd('[CMS] html app');

        this.project.endChanges();

        return fso;
    };

    Application.prototype.backendUrl = function backend(action) {
        return this.config.url.backend + '?action=' + action;
    };

    Application.prototype.request = function request(action, data, dataType) {
        dataType = dataType || 'json';
        data = data || {};
        return $.ajax({
            url: this.backendUrl(action),
            type: 'post',
            dataType: dataType,
            data: data
        });
    };

    Application.prototype.chunkedRequest = function chunkedRequest(action, data) {
        return this.request(action, data);
    };

    function fileInfo(path) {
        path = path.replace(/[\/\\]+/g, '/') || '';

        var info = {
            ext: '',
            name: '',
            path: ''
        };

        var dot = path.lastIndexOf('.');
        var separator = path.lastIndexOf('/');

        if (dot !== -1) {
            info.ext = path.substring(dot + 1).toLowerCase();
        }

        if (separator === -1) {
            info.name = path;
        } else {
            info.name = path.substring(separator + 1);
            info.path = path.substr(0, separator);
        }

        return info;
    }

    return Application;

})();

//